'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.support = exports.supportedLogic = exports.supportedConditions = exports.supportedStates = exports.supportedActions = undefined;

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _supportedStates;

exports.defaultStates = defaultStates;
exports.formatConditionals = formatConditionals;
exports.checkConditionals = checkConditionals;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var supportedActions = exports.supportedActions = {
  visible: 'visible',
  enabled: 'enabled',
  required: 'required'
};

var supportedStates = exports.supportedStates = (_supportedStates = {}, (0, _defineProperty3.default)(_supportedStates, supportedActions.visible, supportedActions.visible), (0, _defineProperty3.default)(_supportedStates, 'invisible', 'invisible'), (0, _defineProperty3.default)(_supportedStates, supportedActions.enabled, supportedActions.enabled), (0, _defineProperty3.default)(_supportedStates, 'disabled', 'disabled'), (0, _defineProperty3.default)(_supportedStates, supportedActions.required, supportedActions.required), (0, _defineProperty3.default)(_supportedStates, 'optional', 'optional'), _supportedStates);

var supportedConditions = exports.supportedConditions = {
  empty: 'empty',
  filled: 'filled',
  checked: 'checked',
  unchecked: 'unchecked',
  value: 'value'
};

var supportedLogic = exports.supportedLogic = {
  and: 'and',
  or: 'or'
};

var support = exports.support = {
  actions: supportedActions,
  states: supportedStates,
  conditions: supportedConditions,
  logic: supportedLogic
};

/**
 * @deprecated We don't use state anymore! Use store.
 */
function defaultStates(field) {
  var _ref;

  return _ref = {}, (0, _defineProperty3.default)(_ref, supportedActions.visible, true), (0, _defineProperty3.default)(_ref, supportedActions.required, field['#required'] || false), (0, _defineProperty3.default)(_ref, supportedActions.enabled, true), _ref;
}

/**
 * Method to format a states object from Drupal, into a more JavaScript friendly format.
 * @param {object} states States object from Drupal field.
 * @returns {boolean|Array} Returns array with formatted conditionals or false if no conditionals found.
 *
 * Example
 *   Old:
 *   {
 *    visible: [
 *      {
 *        ':input[name="name"]': {
 *          empty: true
 *        }
 *      },
 *      {
 *        ':input[name="name"]': {
 *          filled: true
 *        }
 *      }
 *    ]
 *   }
 *   New:
 *   [
 *    {
 *      action: 'visible',
 *      logic: 'and',
 *      conditions: [
 *        {
 *          key: 'name',
 *          condition: 'empty',
 *          value: true
 *        },
 *        {
 *          key: 'name',
 *          condition: 'visible',
 *          value: true
 *        }
 *     ]
 *    }
 *   ]
 */
function formatConditionals() {
  var states = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

  if (!states) {
    return false;
  }

  var mappedStates = (0, _keys2.default)(states).map(function (stateKey) {
    // stateKey, e.g. visible.
    if (!supportedStates[stateKey]) {
      return false; // Don't format conditional if state isn't supported.
    }

    var conditionalKeys = (0, _keys2.default)(states[stateKey]);
    var conditionLogic = states[stateKey][conditionalKeys[1]] === supportedLogic.or ? supportedLogic.or : supportedLogic.and; // Default conditional logic is and.

    var mappedConditions = conditionalKeys.map(function (conditionalKey2) {
      // e.g. ':input[name="name"]'.
      var conditionalKey = conditionalKey2;
      var conditionObject = states[stateKey][conditionalKey];

      if (conditionLogic === supportedLogic.or) {
        // If conditional logic is set to 'or'.
        if (conditionObject === supportedLogic.or) {
          return false; // Don't format the 'or' item.
        }

        conditionalKey = (0, _keys2.default)(conditionObject)[0];
        conditionObject = conditionObject[(0, _keys2.default)(conditionObject)[0]]; // Remove one level of nesting when logic is set to 'or'.
      }

      var dependencyKeyRegex = conditionalKey.match(/name="(\S+)"/);
      if (!dependencyKeyRegex) {
        return false;
      }
      var formattedDependencyKey = dependencyKeyRegex[1]; // Field key of dependency, e.g. 'name' in above example.
      var condition = (0, _keys2.default)(conditionObject)[0]; // e.g. empty.
      var conditionValue = conditionObject[condition]; // e.g. true.

      if (!supportedConditions[condition]) {
        return false; // Don't format conditional if condition isn't supported.
      }

      return {
        key: formattedDependencyKey,
        condition: condition,
        value: conditionValue
      };
    });

    var filteredConditions = mappedConditions.filter(function (c) {
      return c;
    }); // Filter out all 'false' values.

    if (!filteredConditions.length) {
      return false; // Return false when there are no valid conditions found.
    }

    return {
      action: stateKey,
      logic: conditionLogic,
      conditions: filteredConditions
    };
  });

  var filteredStates = mappedStates.filter(function (s) {
    return s;
  }); // Filter out all 'false' values.

  return filteredStates.length ? filteredStates : false; // Return false when there are no valid states found.
}

function formatNewStates(newStates) {
  var formattedStates = {};

  (0, _keys2.default)(newStates).forEach(function (stateKey) {
    var condition = newStates[stateKey];
    switch (stateKey) {
      case supportedStates.visible:
      case supportedStates.enabled:
      case supportedStates.required:
        formattedStates[stateKey] = condition;
        break;
      case supportedStates.invisible:
        formattedStates[supportedActions.visible] = !condition;
        break;
      case supportedStates.disabled:
        formattedStates[supportedActions.enabled] = !condition;
        break;
      case supportedStates.optional:
        formattedStates[supportedActions.required] = !condition;
        break;
      default:
        break;
    }
  });

  return formattedStates;
}

function checkConditionals(formStore, fieldKey) {
  if (!fieldKey) {
    console.error('Conditionals were checked without a fieldKey!');
    return {};
  }

  var field = formStore.getField(fieldKey);

  if (!field || !field.conditionals) {
    // When the form is still being built, not all fields are available.
    return {};
  }

  var newStates = {};

  /**
   * conditional example:
   *  {
   *    action: 'visible',
   *    logic: 'and',
   *    conditions: [condition, condition]
   *  }
   */
  // Reduce conditionals to true or false value, based on conditionals and logic.
  field.conditionals.forEach(function (conditional) {
    /**
     * condition example:
     *  {
       *    key: 'name',
       *    condition: 'empty',
       *    value: true
       *  }
     */
    // Go through conditions per conditional.
    newStates[conditional.action] = conditional.conditions.reduce(function (prevOutcome, condition) {
      var conditionalOutcome = false;

      var dependency = formStore.getField(condition.key);
      if (!dependency) {
        console.error('Cant find dependency ' + condition.key + ' in the conditionals of ' + fieldKey);
        return false;
      }
      var dependencyValue = dependency.value;

      // See what the action of the condition should be.
      switch (condition.condition) {
        case supportedConditions.empty:
        case supportedConditions.filled:
          {
            var isEmpty = dependency.isEmpty;
            var check = condition.condition === supportedConditions.empty ? isEmpty : !isEmpty;
            conditionalOutcome = check;
            break;
          }
        case supportedConditions.checked:
        case supportedConditions.unchecked:
          {
            // When dependencyValue is true, then it is checked.
            var _check = condition.condition === supportedConditions.checked;
            conditionalOutcome = _check === (dependencyValue === true || dependencyValue === '1' || dependencyValue === 'true');
            break;
          }
        case supportedConditions.value:
          {
            // Check if value matches condition.
            conditionalOutcome = dependencyValue === condition.value;
            break;
          }
        default:
          {
            break;
          }
      }

      if (conditional.logic === supportedLogic.and) {
        return prevOutcome && conditionalOutcome;
      } else if (conditional.logic === supportedLogic.or) {
        return prevOutcome || conditionalOutcome;
      }
      return true;
    }, conditional.logic === supportedLogic.and);
  });

  return formatNewStates(newStates);
}